Package: MethylMix
Title: MethylMix: Identifying methylation driven cancer genes
Version: 2.40.0
Description: MethylMix is an algorithm implemented to identify hyper
                                and hypomethylated genes for a disease. MethylMix is based on a
                                beta mixture model to identify methylation states and compares
                                them with the normal DNA methylation state. MethylMix uses a
                                novel statistic, the Differential Methylation value or DM-value
                                defined as the difference of a methylation state with the
                                normal methylation state. Finally, matched gene expression data
                                is used to identify, besides differential, functional
                                methylation states by focusing on methylation changes that
                                effect gene expression. References: 
                                Gevaert 0. MethylMix: an R package for identifying DNA methylation-driven genes. Bioinformatics (Oxford, England). 2015;31(11):1839-41. doi:10.1093/bioinformatics/btv020.
                                Gevaert O, Tibshirani R, Plevritis SK. Pancancer analysis of DNA methylation-driven genes using MethylMix. Genome Biology. 2015;16(1):17. doi:10.1186/s13059-014-0579-8.
Depends: R (>= 3.2.0)
License: GPL-2
Encoding: UTF-8
LazyData: true
Author: Olivier Gevaert
Maintainer: Olivier Gevaert <olivier.gevaert@gmail.com>
Type: Package
Date: 2018-07-13
Imports: foreach, RPMM, RColorBrewer, ggplot2, RCurl, impute,
        data.table, limma, R.matlab, digest
Suggests: BiocStyle, doParallel, testthat, knitr, rmarkdown
biocViews:
        DNAMethylation,StatisticalMethod,DifferentialMethylation,GeneRegulation,GeneExpression,MethylationArray,DifferentialExpression,Pathways,Network
RoxygenNote: 6.0.1
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/MethylMix
git_branch: RELEASE_3_22
git_last_commit: b1e6c56
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
NeedsCompilation: no
Packaged: 2025-10-30 05:00:59 UTC; biocbuild
