# Generated by roxygen2: do not edit by hand

export(ClusterProbes)
export(Download_DNAmethylation)
export(Download_GeneExpression)
export(GetData)
export(MethylMix)
export(MethylMix_ModelGeneExpression)
export(MethylMix_PlotModel)
export(MethylMix_Predict)
export(Preprocess_DNAmethylation)
export(Preprocess_GeneExpression)
import(ggplot2)
importFrom(foreach,"%dopar%")
importFrom(grDevices,dev.off)
importFrom(grDevices,pdf)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,title)
importFrom(stats,anova)
importFrom(stats,aov)
importFrom(stats,as.dist)
importFrom(stats,cor)
importFrom(stats,cutree)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,dnorm)
importFrom(stats,hclust)
importFrom(stats,lm)
importFrom(stats,optim)
importFrom(stats,prcomp)
importFrom(stats,qqline)
importFrom(stats,qqnorm)
importFrom(stats,qqplot)
importFrom(stats,quantile)
importFrom(stats,rgamma)
importFrom(stats,t.test)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,download.file)
importFrom(utils,read.csv)
importFrom(utils,tail)
importFrom(utils,untar)
importFrom(utils,write.table)
