% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{BatchData}
\alias{BatchData}
\title{BatchData data set}
\description{
Data set with batch number for TCGA samples.
}
\keyword{datasets}
