% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dataDoc.R
\docType{data}
\name{SNPprobes}
\alias{SNPprobes}
\title{SNPprobes data set}
\description{
Vector with probes with SNPs.
}
\keyword{datasets}
